unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, OpenGLContext, Forms, Controls, Graphics,
  Dialogs, ExtCtrls, StdCtrls, gl, glu, Types;


const
  DEFAULT_DISTANCE = 5;
  DEFAULT_ROTANGLE_X = 20;
  DEFAULT_ROTANGLE_Y = 30;

type
  { TForm1 }

  TForm1 = class(TForm)
    cbAxes: TCheckBox;
    Label1: TLabel;
    OpenGLControl: TOpenGLControl;
    Panel1: TPanel;
    procedure cbAxesChange(Sender: TObject);
    procedure cbWireFrameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OpenGLControlMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure OpenGLControlMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure OpenGLControlMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure OpenGLControlPaint(Sender: TObject);
    procedure OpenGLControlResize(Sender: TObject);
  private
    { private declarations }
    FRotAngleX, FRotAngleY: Double;
    FShiftX, FShiftY: Double;
    FDistance: Double;
    FMouseDownPt: TPoint;
    FPrevMousePt: TPoint;
    FGLInitDone: Boolean;
    FSceneInitDone: boolean;
    procedure ApplyProjection;

    procedure DrawAxes;
    procedure DrawScene;   // put your own OpenGL code in here
    procedure InitGL;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  Math;


{ TForm1 }

procedure TForm1.ApplyProjection;
const
  VIEWANGLE = 40;
  NEAR_CLIPDIST = 0.1;
  FAR_CLIPDIST = 100;
var
  w, h: Integer;
  d: Double;
begin
  w := OpenGLControl.Width;
  h := OpenGLControl.Height;
  if h = 0 then h := 1;

  // Set viewport to window dimensions
  glViewport(0, 0, w, h);

  // Reset coordinate system
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;

  // Create perspective projection
  gluPerspective(
    VIEWANGLE,      // field of view (degrees) in y direction
    w/h,            // aspect ratio
    NEAR_CLIPDIST,  // distance to near clipping plane
    FAR_CLIPDIST    // distance to far clipping plane
  );

  // Aktuelle Matrix auf ModelView zurücksetzen
  glMatrixMode(GL_MODELVIEW);
end;


procedure TForm1.cbAxesChange(Sender: TObject);
begin
  OpenGLControl.Invalidate;
end;

procedure TForm1.cbWireFrameChange(Sender: TObject);
begin
  FSceneInitDone := false;  // Need a new spacing
  OpenGLControl.Invalidate;
end;


procedure TForm1.DrawAxes;
const
  EPS = 1e-3;
  AXIS_LENGTH = 100;
var
  x0, y0, z0: GLFloat;
begin
  glLineWidth(3);
  glEnable(GL_BLEND);  // for line smoothing

  glColor3f(1, 0, 0);   // x axis --> red
  glBegin(GL_LINES);
    glVertex3f(0, EPS, 0);
    glVertex3f(AXIS_LENGTH, EPS, 0);
  glEnd;

  glColor3f(0, 0, 1);  // OpenGL y axis (= Model z axis) --> blue
  glBegin(GL_LINES);
    glVertex3f(0, EPS, 0);
    glVertex3f(0, EPS, AXIS_LENGTH);
  glEnd;

  glColor3f(0, 1, 0);  // OpenGL z axis (= Model y axis) green
  glBegin(GL_LINES);
    glVertex3f(0, EPS, 0);
    glVertex3f(0, AXIS_LENGTH, 0);
  glEnd;

  glDisable(GL_BLEND);
end;


procedure TForm1.DrawScene;
begin
  glBegin(GL_POLYGON);
    glColor3f(1, 0, 0);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(1.0, 1.0, 1.0);
    glVertex3f(-1.0, 1.0, 1.0);
    glVertex3f(-1.0, -1.0, 1.0);
    glVertex3f(1.0, -1.0, 1.0);
  glEnd;

  glBegin(GL_POLYGON);
    glColor3f(0, 1, 0);
    glNormal3f(0.0, 0.0, -1.0);
    glVertex3f(1.0, 1.0, -1.0);
    glVertex3f(1.0, -1.0, -1.0);
    glVertex3f(-1.0, -1.0, -1.0);
    glVertex3f(-1.0, 1.0, -1.0);
  glEnd;

  glBegin(GL_POLYGON);
    glColor3f(0, 0, 1);
    glNormal3f(-1.0, 0.0, 0.0);
    glVertex3f(-1.0, 1.0, 1.0);
    glVertex3f(-1.0, 1.0, -1.0);
    glVertex3f(-1.0, -1.0, -1.0);
    glVertex3f(-1.0, -1.0, 1.0);
  glEnd;

  glBegin(GL_POLYGON);
    glColor3f(1, 1, 0);
    glNormal3f(1.0, 0.0, 0.0);
    glVertex3f(1.0, 1.0, 1.0);
    glVertex3f(1.0, -1.0, 1.0);
    glVertex3f(1.0, -1.0, -1.0);
    glVertex3f(1.0, 1.0, -1.0);
  glEnd;

  glBegin(GL_POLYGON);
    glColor3f(1, 0, 1);
    glNormal3f(0.0, 1.0, 0.0);
    glVertex3f(-1.0, 1.0, -1.0);
    glVertex3f(-1.0, 1.0, 1.0);
    glVertex3f(1.0, 1.0, 1.0);
    glVertex3f(1.0, 1.0, -1.0);
  glEnd;

  glBegin(GL_POLYGON);
    glColor3f(0, 1, 1);
    glNormal3f(0.0, -1.0, 0.0);
    glVertex3f(-1.0, -1.0, -1.0);
    glVertex3f(1.0, -1.0, -1.0);
    glVertex3f(1.0, -1.0, 1.0);
    glVertex3f(-1.0, -1.0, 1.0);
  glEnd;
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  FDistance := DEFAULT_DISTANCE;
  FRotAngleX := DEFAULT_ROTANGLE_X;
  FRotAngleY := DEFAULT_ROTANGLE_Y;
end;


procedure TForm1.InitGL;
begin
  if FGlInitDone then
    exit;

  FGLInitDone := true;

  glClearColor(0.5, 0.5 ,0.5, 1.0);   // sets background color
  glClearDepth(1.0);
  glDepthFunc(GL_LEQUAL);             // the type of depth test to do
  glEnable(GL_DEPTH_TEST);            // enables depth testing
  glShadeModel(GL_SMOOTH);            // enables smooth color shading

  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
  glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
end;


procedure TForm1.OpenGLControlMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FMouseDownPt := Point(X, Y);
  FPrevMousePt := FMouseDownPt;
end;


procedure TForm1.OpenGLControlMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
const
  SPEED = 0.5;
  DELTA = 0.003;
var
  f: Double;
begin
  f := 1.0;
  if ssShift in Shift then
    f := f * 5.0;
  if ssLeft in Shift then begin
    if FMouseDownPt.Y > OpenGLControl.Height div 2 then begin
      FRotAngleY := FRotAngleY + (X - FPrevMousePt.X)*SPEED*f;
      FRotAngleX := FRotAngleX + (Y - FPrevMousePt.Y)*SPEED*f;
    end
    else begin
      FRotAngleY := FRotAngleY - (X - FPrevMousePt.X)*SPEED*f;
      FRotAngleX := FRotAngleX - (Y - FPrevMousePt.Y)*SPEED*f
    end;
    FPrevMousePt := Point(X, Y);
    OpenGLControl.Invalidate;
  end else
  if ssRight in Shift then begin
    FShiftX := FShiftX + (x - FPrevMousePt.X)*DELTA*f;
    FShiftY := FShiftY + (FPrevMousePt.Y - y)*DELTA*f;
    FPrevMousePt := Point(X, Y);
    OpenGLControl.Invalidate;
  end;
end;


procedure TForm1.OpenGLControlMouseWheel(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
const
  SPEED = 0.0005;
var
  f: Double;
begin
  if (ssCtrl in Shift) then
    f := -1.0
  else
    f := 1.0;
  if (ssShift in Shift) then
    f := f * 2;
  FDistance := FDistance * (1.0 + f * WheelDelta * SPEED);
  OpenGLControl.Invalidate;
end;


procedure TForm1.OpenGLControlPaint(Sender: TObject);
begin
  if not OpenGLControl.MakeCurrent then
    exit;

  if not FSceneInitDone then begin
    InitGL;
    ApplyProjection;
    FSceneInitDone := true;
  end;

  glMatrixMode(GL_MODELVIEW);

  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity;             { clear the modelview matrix }

  { viewing transformation }
  glTranslatef(FDistance*FShiftX, FDistance*FShiftY, -FDistance);
  glRotatef(FRotAngleX, 1, 0, 0);
  glRotatef(FRotAngleY, 0, 1, 0);

  DrawScene;

  if cbAxes.Checked then
    DrawAxes;

  OpenGLControl.SwapBuffers;
end;


procedure TForm1.OpenGLControlResize(Sender: TObject);
begin
  if FSceneInitDone and OpenGLControl.MakeCurrent then
    ApplyProjection;
end;


end.

